<?php

namespace Drupal\easy_google_analytics_counter;

use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerAwareTrait;
use Drupal\Core\Database\Query\Condition;
use Throwable;
// --------- Use from google analytics library ----------.
use Google_Client;
use Google_Service_AnalyticsReporting;
use Google_Service_AnalyticsReporting_GetReportsRequest;
use Google_Service_AnalyticsReporting_GetReportsResponse;

/**
 * Class ConnectionService.
 */
class ConnectionService implements ContainerAwareInterface, ConnectionServiceInterface {

  use ContainerAwareTrait;

  /**
   * Drupal\Core\Config\ConfigFactoryInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * Drupal\Core\Logger\LoggerChannelFactoryInterface definition.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $logger;

  /**
   * Google_Service_AnalyticsReporting definition.
   *
   * @var \Google_Service_AnalyticsReporting
   */
  protected $analytics;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Initialize the google analytics service.
   *
   * @throws \Google_Exception
   *   Throw if file not exists.
   */
  protected function initializeAnalytics() {
    // Use the developers console and download your service account
    // credentials in JSON format. Place them in this directory or
    // change the key file location if necessary.
    // $key_file_location = __DIR__ . '/service-account-credentials.json';
    // Create and configure a new client object.
    $client = new Google_Client();
    $client->setApplicationName($this->config->get('application_name'));
    $client->setAuthConfig($this->getKeyLocation());
    $client->setScopes(['https://www.googleapis.com/auth/analytics.readonly']);

    $this->analytics = new Google_Service_AnalyticsReporting($client);
  }

  /**
   * {@inheritdoc}
   */
  public function request(string $page_path = '') {
    // Set services.
    $this->config = $this->container->get('config.factory')
      ->get('easy_google_analytics_counter.admin');
    $this->logger = $this->container->get('logger.factory');
    $this->moduleHandler = $this->container->get('module_handler');

    try {
      // Initialize analytics.
      $this->initializeAnalytics();
    }
    catch (Throwable $e) {
      $this->logger
        ->get('easy_google_analytics_counter')
        ->error('Process update page views from google analytics stopped. ' . $e->getMessage());
      return;
    }

    // Your view ID, defined on admin form.
    $view_id = $this->config->get('view_id');

    // Prepare google dimensions.
    $dimensions = ['name' => 'ga:pagepath'];
    $dimension = $this->config->get('sort_dimension');
    if (strtolower($dimension) !== 'ga:pageviews') {
      $dimensions[] = ['name' => $dimension];
    }

    // Create DateRange.
    $days = $this->config->get('start_date');
    $start = strtotime("-$days day");
    $start_date = date('Y-m-d', $start);
    $end_date = date('Y-m-d');

    $query = [
      'viewId' => $view_id,
      'dateRanges' => [
        'startDate' => $start_date,
        'endDate' => $end_date,
      ],
      'metrics' => [
        'expression' => 'ga:pageviews',
      ],
      'dimensions' => $dimensions,
      'orderBys' => [
        'fieldName' => 'ga:pageviews',
        'sortOrder' => 'DESCENDING',
      ],
      'pageSize' => $this->config->get('number_items'),
      'includeEmptyRows' => FALSE,
      'hideTotals' => TRUE,
    ];

    // Add page path filter.
    if (!empty($page_path)) {
      $query['dimensionFilterClauses'] = [
        'filters' => [
          'dimension_name' => 'ga:pagepath',
          'operator' => 'ENDS_WITH',
          'expressions' => $page_path,
        ],
      ];
    }
    // Allow modules to query altering.
    $this->moduleHandler->alter('easy_google_analytics_counter_query', $query);

    // Build the request and response.
    $body = new Google_Service_AnalyticsReporting_GetReportsRequest();
    $body->setReportRequests([$query]);
    // Allow modules to body altering.
    $this->moduleHandler->alter('easy_google_analytics_counter_body', $body);

    try {
      $response = $this->analytics->reports->batchGet($body);
      $this->setData($response);
    }
    catch (Throwable $e) {
      $this->logger
        ->get('easy_google_analytics_counter')
        ->error($e->getMessage());
    }

  }

  /**
   * Set pageviews data to database.
   *
   * @param \Google_Service_AnalyticsReporting_GetReportsResponse $response
   *   The response from GA.
   */
  protected function setData(Google_Service_AnalyticsReporting_GetReportsResponse $response) {
    $ga_views = [];
    $fp = NULL;
    $debug = $this->config->get('debug');
    $reports = $response->getReports();
    $alias_manager = $this->container->get('path_alias.manager');
    if ($debug === 1) {
      $url = $this->container->get('file_system')
        ->realpath('public://ga_alias_file.csv');
      $fp = fopen($url, 'wb');
      fputcsv($fp, ['Page views', 'GA path', 'System path']);
    }
    /** @var \Google_Service_AnalyticsReporting_Report $report */
    foreach ($reports as $reportIndex => $report) {
      $header = $report->getColumnHeader();
      $dimensionHeaders = $header->getDimensions();
      $rows = $report->getData()->getRows();
      $matches = [];
      /** @var \Google_Service_AnalyticsReporting_ReportRow $row */
      foreach ($rows as $row) {
        $dimensions = $row->getDimensions();
        $metrics = $row->getMetrics();
        foreach ($dimensionHeaders as $i => $dimensionheader) {
          if (strtolower($dimensionheader) === 'ga:pagepath') {
            $alias = $alias_manager
              ->getPathByAlias(strstr($dimensions[$i], '?', 1));
            if (empty($alias)) {
              $alias = $alias_manager->getPathByAlias($dimensions[$i]);
            }
            $values = reset($metrics)->getValues();
            $page_views = $values[0] ?? '';
            if (preg_match('/node\/(\d+)(?<!\/)$/', $alias, $matches) &&
              !empty($matches[1]) && !empty($page_views)) {
              // Merge different aliases for same node id.
              if (!isset($ga_views[$matches[1]])) {
                $ga_views[$matches[1]] = $page_views;
              }
              else {
                $ga_views[$matches[1]] += $page_views;
              }
            }
            if ($fp !== NULL) {
              // Put data to csv file.
              fputcsv($fp, [$page_views, $dimensions[$i], $alias]);
            }
          }
        }
      }
    }

    // Write results to database.
    $this->updateNodePageViews($ga_views);

    // Results logging.
    $context = [
      '%count' => count($ga_views),
      '%elements' => http_build_query($ga_views, ' ', ';'),
    ];
    $this->logger
      ->get('easy_google_analytics_counter')
      ->notice('updated %count elements page views from Google Analytics: %elements', $context);

    if ($fp !== NULL) {
      // Close csv file.
      fclose($fp);
    }
  }

  /**
   * Write results to database.
   *
   * @param array $ga_views
   *   The node page views array.
   */
  protected function updateNodePageViews(array $ga_views) {
    $update_nids = [];
    $database = $this->container->get('database');
    foreach ($ga_views as $nid => $value) {
      $or = new Condition('OR');
      $or->condition('page_views', $value, '>')
        ->isNull('page_views');
      $updated = $database->update('node_field_data')
        ->fields(['page_views' => $value])
        ->condition('nid', $nid)
        ->condition($or)
        ->execute();
      if ($updated > 0) {
        $update_nids[$nid] = $value;
      }
    }
    if (!empty($update_nids)) {
      // Invalidate the EGAC page views cache tag,
      $this->container->get('cache_tags.invalidator')
        ->invalidateTags(['easy_google_analytics_counter_page_views']);
      // Let other modules to use google aanalytics views result.
      $this->moduleHandler
        ->invokeAll('easy_google_analytics_counter_update_node_page_views', [$update_nids]);
    }
  }

  /**
   * Return key file location.
   *
   * @return string
   *   The url of key file.
   */
  protected function getKeyLocation(): string {
    $url = $this->config->get('service_account_credentials_json_path');
    if (!$url) {
      $files = $this->config->get('service_account_credentials_json');
      /** @var \Drupal\file\FileInterface $file */
      if (!empty($files[0]) &&
        $file = $this->container->get('entity_type.manager')
          ->getStorage('file')->load($files[0])) {
        $url = $this->container->get('file_system')
          ->realpath($file->getFileUri());
      }
    }

    return $url;
  }

}
